from . import contrast_manager
from . import factorial_design
from .image_calculator import ImageCalculator
from .model_estimation import ModelEstimation
from .spm_object import SPMObject
from .stats import Stats
from .utils import *
